import os.path
import re
import shutil
from abc import ABC, abstractmethod
from typing import Any, Dict, Tuple

from boto3 import Session

from dsdlsdk.exception.exception import DatasetPathNotExists


class Storage(ABC):
    """
    对config里支持的本地存储方式的一种抽象
    """

    @abstractmethod
    def remove_tree(self, path: str) -> bool:
        """
        删除整个目录
        Returns:

        """
        pass


class LocalDiskStorage(Storage):

    def remove_tree(self, local_path: str) -> bool:
        if not os.path.exists(local_path):
            raise DatasetPathNotExists(f"local path {local_path} not exist")
        try:
            shutil.rmtree(local_path, ignore_errors=True)
            return True
        except Exception as e:
            return False


class S3Storage(Storage):

    def __init__(self, path: str, endpoint: str, access_key: str,
                 secret_key: str):
        self.path = path
        self.bucket, _ = self.__split_s3_path(path)
        self.endpoint = endpoint
        self.access_key = access_key
        self.secret_key = secret_key
        # build s3 client
        self.s3cli = self.__build_s3_cli()

    def remove_tree(self, s3_path: str) -> bool:
        bucket, obj_key = self.__split_s3_path(s3_path)
        for files in self.list_objects(bucket, obj_key):
            self.s3cli.delete_object(Bucket=bucket, Key=files['Key'])

        return True

    def __build_s3_cli(self):
        session = Session(self.access_key, self.secret_key)
        s3_client = session.client("s3",
                                   endpoint_url=self.endpoint,
                                   use_ssl=False)
        return s3_client

    def __split_s3_path(self, s3_path: str) -> Tuple[str, str]:
        """

        Args:
            s3_path:

        Returns:

        """
        s3_path_pattern = re.compile("^s3://([^/]+)(?:/(.*))?$")
        m = s3_path_pattern.match(s3_path)
        if m is None:
            return "", ""
        return m.group(1), (m.group(2) or "")

    def list_objects(self, bucket, prefix, delimiter=''):
        """

        Args:
            bucket:
            prefix:
            delimiter:

        Returns:

        """
        marker = None
        while True:
            list_kwargs = dict(MaxKeys=1000, Bucket=bucket, Prefix=prefix)
            if marker:
                list_kwargs['Marker'] = marker
            response = self.s3cli.list_objects(**list_kwargs)
            contents = response.get("Contents", [])
            yield from contents
            if not response.get("IsTruncated") or len(contents) == 0:
                break
            marker = contents[-1]['Key']


class SftpStorage(Storage):

    def remove_tree(self, path: str) -> bool:
        raise NotImplementedError

    def __init__(self):
        raise NotImplementedError



"""

References:
    https://developer.aliyun.com/article/740160
    https://docs.python.org/zh-cn/3/library/argparse.html
"""

import importlib
import inspect
import json
import os.path
import sys
from pathlib import Path
from typing import Any

from loguru import logger

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

from commands.__init__ import __version__
from commands.cmdbase import CmdBase
from commands.const import (
    DEFAULT_CLI_CONFIG_FILE,
    DEFAULT_CLI_LOG_FILE_PATH,
    DEFAULT_CONFIG_DIR,
    DEFAULT_LOCAL_STORAGE_PATH,
    PROG_NAME,
    SQLITE_DB_PATH,
)
from commons.argument_parser import CustomHelpFormatter
from commons.argument_parser import DsdlArgumentParser as ArgumentParser
from commons.loghandler import setup_loger
from utils.admin import initialize_db

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


class DSDLClient(object):
    """
    DSDL 命令行客户端
    """

    def __init__(self) -> None:
        """
        初始化命令行客户端
        """
        self.__config = self.__init_cli_config()
        self.__parser = ArgumentParser(
            prog=PROG_NAME,
            description="""
            Use '%(prog)s <command>' to access/load datasets either from local
            file system or remote cloud storage, can also perform data-preprocessing
            including filtering, visualization, etc.""",
            epilog=
            "Report bugs to https://github.com/opendatalab/dsdl-sdk/issues",
            usage=
            f"{PROG_NAME} GLOBAL_FLAGS | COMMAND [COMMAND_ARGS] [DATASET_NAME]\n",
            formatter_class=CustomHelpFormatter,
        )
        self.__subparsers = self.__parser.add_subparsers(title="Commands", )
        self.__init_subcommand_parser()

        self.__init_global_flags()  # 初始化全局参数
        self.__args = self.__parser.parse_args()

    def execute(self):
        """
        执行命令
        Returns:

        """
        if hasattr(self.__args, "command_handler"):
            self.__args.command_handler(self.__args, self.__config)
        else:
            self.__parser.print_usage()

    def __init_cli_config(self) -> Any:
        """
        TODO 初始化命令行配置

        Returns:

        """
        # 检查配置文件是否存在，如果没有则创建一个
        # 检查sqlite数据库是否存在，如果没有则创建一个
        # 检查sqlite数据库是否有表，如果没有则创建表
        if os.path.exists(DEFAULT_CONFIG_DIR) is False:  # 配置目录
            os.makedirs(DEFAULT_CONFIG_DIR)
        if os.path.exists(DEFAULT_CLI_LOG_FILE_PATH) is False:
            os.makedirs(DEFAULT_CLI_LOG_FILE_PATH)
        if os.path.exists(SQLITE_DB_PATH) is False:  # sqlite数据库,并初始化table
            initialize_db(SQLITE_DB_PATH)
        if os.path.exists(DEFAULT_CLI_CONFIG_FILE) is False:  # 默认配置文件
            with open(DEFAULT_CLI_CONFIG_FILE, 'w') as f:
                with open(
                        os.path.join(os.path.dirname(__file__),
                                     'resources/default_cli_cfg.json'),
                        'r') as f2:
                    x = json.loads(f2.read())
                    x["storage"]['default'][
                        'path'] = DEFAULT_LOCAL_STORAGE_PATH
                    f.write(json.dumps(x, ensure_ascii=False, indent=4))
        if os.path.exists(DEFAULT_LOCAL_STORAGE_PATH) is False:  # 默认存放数据的目录
            os.makedirs(DEFAULT_LOCAL_STORAGE_PATH)

        #########################################################
        # todo 以上正确做法是放在pip install的时候，自定义一个安装脚本，目前先简单做,加快命令速度
        #########################################################

        # 接下来读取配置文件，然后返回成一个json对象
        with open(DEFAULT_CLI_CONFIG_FILE, "r") as f:
            return json.loads(f.read())

    def __init_global_flags(self):
        """
        初始化默认参数
        Returns: None

        """
        self.__parser._optionals.title = "Global flags"
        self.__parser.add_argument(
            "-v",
            "--version",
            action="version",
            version=f"%(prog)s {__version__}",
            help="show the %(prog)s version and exit.",
        )

    def __init_subcommand_parser(self) -> None:
        """
        此处初始化子命令解析器。

        过程是扫描commands目录里集成了CmdBase的子类，然后
        Args:
            subcommand:
            help:

        Returns:

        """
        import commands

        pkgs = [
            module.stem for module in Path(commands.__path__[0]).iterdir()
            if module.is_file() and module.suffix == '.py'
            and not module.name.startswith('_')
        ]  # 获取commands目录下的所有py文件
        for pkg in pkgs:  # TODO这里的性能是硬性最大的地方，可以通过缓存或者打包的时候预定义方式优化
            module = importlib.import_module(f'commands.{pkg}')
            for clz_name, clz_obj in inspect.getmembers(module):
                if inspect.isclass(clz_obj) and issubclass(
                        clz_obj, CmdBase) and not inspect.isabstract(clz_obj):
                    cmd_clz = clz_obj()
                    subcmd_parser = cmd_clz.setup_parser(self.__subparsers)
                    subcmd_parser.set_defaults(
                        command_handler=cmd_clz.cmd_main)
                    subcmd_parser._optionals.title = "Command args"
                    subcmd_parser._positionals.title = "Positional arguments"
                    subcmd_parser.formatter_class = CustomHelpFormatter


def main():
    setup_loger()
    logger.remove(0)
    DSDLClient().execute()


if __name__ == "__main__":
    setup_loger()
    DSDLClient().execute()



"""
一个命令的实现例子

Examples:
    $python cli.py example --show tell me the truth
    >> Namespace(show=["'tell", 'me', 'the', "truth'"], command_handler=<bound method Example.cmd_entry of <commands.example.Example object at 0x0000017E6FD1DB40>>)
    >> ["'tell", 'me', 'the', "truth'"]
"""
import os

from commands.cmdbase import CmdBase
from commands.const import DSDL_CLI_DATASET_NAME
from commons.argument_parser import EnvDefaultVar
from tabulate import tabulate
from rich.tree import Tree
from rich import print as rprint
from utils import admin, query, plot
from utils.oss_ops import ops
import yaml

aws_access_key_id = query.aws_access_key_id
aws_secret_access_key = query.aws_secret_access_key
endpoint_url = query.endpoint_url
region_name = query.region_name
default_bucket = query.default_bucket


class Inspect(CmdBase):
    """
    Example command
    """

    def init_parser(self, subparsers):
        """
        Initialize the parser for the command
        document : https://docs.python.org/zh-cn/3/library/argparse.html#

        Args:
            subparsers:

        Returns:

        """
        inspect_parser = subparsers.add_parser(
            "inspect",
            help="Inspect dataset info",
            example="inspect.example",
            description="Inspect dataset info",
        )  # example 样例文件位于resources/下，普通的文本文件，每个命令写一个

        inspect_parser.add_argument(
            "dataset_name",
            action=EnvDefaultVar,
            envvar=DSDL_CLI_DATASET_NAME,
            type=str,
            help="Dataset name. The arg is optional only when the default dataset name was set by cd command.",
            metavar="",
        )

        inspect_parser.add_argument(
            "--split-name",
            type=str,
            help="The split name of the dataset, such as train/test/validation split.",
            metavar="",
        )

        group = inspect_parser.add_mutually_exclusive_group()

        group.add_argument(
            "-d",
            "--description",
            action="store_true",
            help="The split name of the dataset, such as train/test/unlabeled or user self-defined split.",
        )
        group.add_argument(
            "-s",
            "--statistics",
            action="store_true",
            help="Some statistics of the dataset.",
        )
        group.add_argument(
            "-m",
            "--metadata",
            action="store_true",
            help="Show metadata of the dataset.",
        )
        group.add_argument(
            "--schema",
            action="store_true",
            help="Show schema of the dataset.",
        )
        group.add_argument(
            "--preview",
            action="store_true",
            help="Preview of the dataset.",
        )

        return inspect_parser

    def cmd_entry(self, cmdargs, config, *args, **kwargs):
        """
        Entry point for the command

        Args:
            self:
            cmdargs:
            config:

        Returns:

        """

        dataset_name = cmdargs.dataset_name
        description = cmdargs.description
        statistics = cmdargs.statistics
        schema = cmdargs.schema
        metadata = cmdargs.metadata
        preview = cmdargs.preview
        split_name = cmdargs.split_name

        s3_client = ops.OssClient(
            endpoint_url=endpoint_url,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
        )
        db_client = admin.DBClient()
        remote_dataset_list = [
            x.replace("/", "") for x in s3_client.get_dir_list(default_bucket, "")
        ]

        if dataset_name not in remote_dataset_list:
            print("there is no dataset named %s in remote repo" % dataset_name)
            exit()

        if db_client.is_dataset_local_exist(dataset_name):
            dataset_dict = query.get_dataset_info(dataset_name)
        else:
            parquet_key = "/".join([dataset_name, "parquet", "dataset.yaml"])
            dataset_dict = yaml.safe_load(
                s3_client.read_file(default_bucket, parquet_key)
            )

        if description:
            print("dataset description".center(100, "="))
            print(dataset_dict["dsdl_meta"]["dataset"]["meta"]["description"])

        if statistics:
            print("dataset statistics".center(100, "="))
            print("# " + "basic indicators")
            print(
                tabulate(
                    [dataset_dict["statistics"]["dataset_stat"]],
                    headers="keys",
                    tablefmt="plain",
                )
            )

            plot_list = dataset_dict["statistics"]["plots"]

            for p in plot_list:
                if p["type"] == "bar":
                    name = p["name"]
                    labels = p["data"]["x_data"]
                    y_data = p["data"]["y_data"]
                    data = list(zip(*[x["data"] for x in y_data]))
                    y_categories = [x["name"] for x in y_data]
                    y_colors = [x["color"] for x in y_data]
                    y_label = p["data"]["y_label"]
                    plot.plt_cmd_bar(
                        name, labels, data, y_categories, y_colors, y_label
                    )

        if schema:
            print("dataset schema".center(100, "="))
            schema_dict = dataset_dict["dsdl_meta"]["struct"]
            dsdl_version = schema_dict["$dsdl-version"]
            fields = schema_dict["Sample"]["$fields"]
            optional = schema_dict["Sample"]["$optional"]
            schema_tree = Tree(dataset_name)
            for k, v in fields.items():
                field_str = "%s: %s" % (k, v)
                if k in optional:
                    field_str = field_str + " (Optional)"
                schema_tree.add(field_str)

            print("# dsdl version: " + dsdl_version)
            rprint(schema_tree)

        if metadata:
            print("dataset metadata".center(100, "="))
            dataset_meta = dataset_dict["dsdl_meta"]["dataset"]["meta"]
            for k, v in dataset_meta.items():
                print(k + ": " + v)
            class_dict = dataset_dict["dsdl_meta"]["class_dom"]

            for k, v in class_dict.items():
                if not str(k).startswith("$"):
                    print(k + ":")
                    print(v["classes"])

        if preview:
            print("Previewing the dataset...")
            from utils.views.view import View

            view = View(dataset_name, inspect=True)

            from utils.admin import DBClient

            dbcli = DBClient()
            local_exists = dbcli.is_dataset_local_exist(dataset_name)

            if local_exists is True:
                view.view_from_inspect()
            else:
                print(f"Dataset {dataset_name} is not exists on local.")



"""

cd function

Examples:
    $python cli.py studio coco
    Collecting usage statistics. To deactivate, set browser.gatherUsageStats to False.
    You can now view your Streamlit app in your browser.
    Network URL: http://127.0.0.1:8501
    External URL: http://?.?.?.?:8501
"""


from utils.views.view import View
from commands.cmdbase import CmdBase
from commons.argument_parser import EnvDefaultVar
from utils.admin import DBClient
from commands.const import (
    DEFAULT_CLI_CONFIG_FILE,
    DEFAULT_CLI_LOG_FILE_PATH,
    DEFAULT_CONFIG_DIR,
    DEFAULT_LOCAL_STORAGE_PATH,
    PROG_NAME,
    SQLITE_DB_PATH,
)

import commons.stdio as stdio


class Studio(CmdBase):
    """
    cd command
    """

    def init_parser(self, subparsers):
        """
        Initialize the parser for the command
        document : https://docs.python.org/zh-cn/3/library/argparse.html#

        Args:
            subparsers:

        Returns:

        """
        studio_parser = subparsers.add_parser(
            "studio",
            help="view the specified dataset on webpage.",
            example="studio.example",
        )

        studio_parser.add_argument(
            "dataset_name",
            action=EnvDefaultVar,
            envvar="DSDL_CLI_DATASET_NAME",
            type=str,
            help="dataset name",
            metavar="[DATASET NAME]",
        )

        studio_parser_group = studio_parser.add_mutually_exclusive_group()

        studio_parser_group.add_argument(
            "-l",
            "--local",
            nargs="?",
            default="True",
            const="True",
            type=bool,
            help="view local[default] dataset.",
            metavar="SET to True to VIEW LOCAL DATASET",
        )
        studio_parser_group.add_argument(
            "-r",
            "--remote",
            nargs="?",
            default="False",
            const="False",
            type=bool,
            help="view remote[when explicitly specified to true] dataset.",
            metavar="SET to True(default false) to VIEW REMOTE DATASET",
        )
        return studio_parser

    def cmd_entry(self, cmdargs, config, *args, **kwargs):
        """
        Entry point for the command

        Args:
            self:
            cmdargs:
            config:

        Returns:

        """

        dataset_name = cmdargs.dataset_name
        local = cmdargs.local
        remote = cmdargs.remote

        dbcli = DBClient()
        local_exists = dbcli.is_dataset_local_exist(dataset_name)

        # TODO support remote dataset exists or not in the future:
        # remote_exists = dbcli.is_dataset_remote_exist(dataset_name)

        view = View(dataset_name)

        # print(f"local:{local}")
        # print(f"remote:{str(remote)}")
        # print(f"exists on localhost: {local_exists}")

        if remote is True:
            if local_exists is True:
                use_local = input(
                    f"Dataset {dataset_name} already exists on local.\nDo you want view using local dataset:(y/n)?"
                )
                if use_local in ["y", "Y", "yes", "Yes"]:
                    view.view_local_dataset()
                elif use_local in ["n", "N", "no", "No"]:
                    view.view_remote_dataset()
            else:
                view.view_remote_dataset()
        elif local is True:
            if local_exists is True:
                view.view_local_dataset()
            else:
                stdio.print_stderr(f"Dataset {dataset_name} is not exists on local.")



"""
一个命令的实现例子

Examples:
    $python cli.py example --show tell me the truth
    >> Namespace(show=["'tell", 'me', 'the', "truth'"], command_handler=<bound method Example.cmd_entry of <commands.example.Example object at 0x0000017E6FD1DB40>>)
    >> ["'tell", 'me', 'the', "truth'"]
"""

from commands.cmdbase import CmdBase
from commands.const import DSDL_CLI_DATASET_NAME
from commons.argument_parser import EnvDefaultVar
from commons.stdio import print_stdout


class Example(CmdBase):
    """
    Example command
    """

    def init_parser(self, subparsers):
        """
        Initialize the parser for the command
        document : https://docs.python.org/zh-cn/3/library/argparse.html#

        Args:
            subparsers:

        Returns:

        """
        example_parser = subparsers.add_parser(
            'example',
            help='Show the working tree status',
            example="example.example"
        )  # example 样例文件位于resources/下，普通的文本文件，每个命令写一个
        example_parser.add_argument("-s",
                                    '--show',
                                    nargs='?',
                                    default='SHOW',
                                    help='show example string.....',
                                    metavar='SS')
        example_parser.add_argument('--test1',
                                    nargs='?',
                                    default='x',
                                    help='show test1',
                                    metavar='a')
        example_parser.add_argument('--test-1234',
                                    nargs='?',
                                    default='t',
                                    help='show test-1234',
                                    metavar='c')
        example_parser.add_argument("dataset_name",
                                    action=EnvDefaultVar,
                                    envvar=DSDL_CLI_DATASET_NAME,
                                    nargs=1,
                                    type=str,
                                    help='dataset name',
                                    metavar='[dataset name]')
        return example_parser

    def cmd_entry(self, cmdargs, config, *args, **kwargs):
        """
        Entry point for the command

        Args:
            self:
            cmdargs:
            config:

        Returns:

        """
        print_stdout(cmdargs)
        print_stdout(f"{cmdargs.show}")
        print_stdout(cmdargs.dataset_name)



import os

from commands.const import DEFAULT_CLI_LOG_FILE_PATH
from loguru import logger


def setup_loger():
    # configure log handler only once here
    logger.add(sink = os.path.join(DEFAULT_CLI_LOG_FILE_PATH, 'runtime.log'),
            format = "{time:YYYY-MM-DD HH:mm:ss} | {level} | {module}.{function} : {message}",
            rotation = '1 day' 
            )


"""
一个命令的实现例子

Examples:
    $python cli.py example --show tell me the truth
    >> Namespace(show=["'tell", 'me', 'the', "truth'"], command_handler=<bound method Example.cmd_entry of <commands.example.Example object at 0x0000017E6FD1DB40>>)
    >> ["'tell", 'me', 'the', "truth'"]
"""
import os

import yaml

from commands.cmdbase import CmdBase
from commands.const import DSDL_CLI_DATASET_NAME, DEFAULT_LOCAL_STORAGE_PATH
from commons.argument_parser import EnvDefaultVar
from utils import admin, query
from utils.oss_ops import ops

default_path = DEFAULT_LOCAL_STORAGE_PATH

aws_access_key_id = query.aws_access_key_id
aws_secret_access_key = query.aws_secret_access_key
endpoint_url = query.endpoint_url
region_name = query.region_name
default_bucket = query.default_bucket


class Get(CmdBase):
    """
    Example command
    """

    aws_access_key_id = "ailabminio"
    aws_secret_access_key = "123123123"
    endpoint_url = "10.140.0.94:9800"
    region_name = "ailab"

    def init_parser(self, subparsers):
        """
        Initialize the parser for the command
        document : https://docs.python.org/zh-cn/3/library/argparse.html#

        Args:
            subparsers:

        Returns:

        """
        select_parser = subparsers.add_parser('get', help='download data from repo',
                                              example="get.example",
                                              description='Download data from repo', )

        select_parser.add_argument("dataset_name", action=EnvDefaultVar,
                                   envvar=DSDL_CLI_DATASET_NAME, type=str,
                                   help='Dataset name. The arg is optional only when the default dataset name was set by cd command.',
                                   metavar='')
        select_parser.add_argument("-s", "--split-name", type=str,
                                   help='The split name of the dataset, such as train/test/validation split.',
                                   metavar='')
        # select_parser.add_argument("-p", "--part", type=str,
        #                            help='The part name of the dataset, such as label/media.',
        #                            metavar='')
        select_parser.add_argument("-o", "--output", type=str,
                                   help='Target saving path.',
                                   metavar='')
        select_parser.add_argument("--label", action="store_true",
                                   help='Download label data only.',
                                   )

        return select_parser

    def cmd_entry(self, cmdargs, config, *args, **kwargs):
        """
        Entry point for the command

        Args:
            self:
            cmdargs:
            config:

        Returns:

        """
        conf_dict = admin.get_config_dict()

        # get params
        dataset_name = cmdargs.dataset_name
        split_name = cmdargs.split_name
        output = cmdargs.output if cmdargs.output else 'default'
        output_path = conf_dict['storage'][cmdargs.output]['path'] if cmdargs.output else default_path
        label_flag = cmdargs.label

        # construct class of s3 client
        s3_client = ops.OssClient(endpoint_url=endpoint_url, aws_access_key_id=aws_access_key_id,
                                  aws_secret_access_key=aws_secret_access_key, region_name=region_name)

        # construct class of sqlite client
        db_client = admin.DBClient()

        # get dataset list in remote repo
        if not s3_client.is_dataset_remote_exist(default_bucket, dataset_name):
            print("there is no dataset named %s in remote repo" % dataset_name)
            exit()

        # dataset & split flag of existence
        dataset_exist_flag = db_client.is_dataset_local_exist(dataset_name)
        split_exist_flag = db_client.is_split_local_exist(dataset_name, split_name)

        # get dataset dir/parquet dir/s3 parquet prefix/media dir/s3 media prefix
        dataset_dir = os.path.join(output_path, dataset_name) if not db_client.get_local_dataset_path(
            dataset_name) else db_client.get_local_dataset_path(dataset_name)
        s3_parquet_prefix = dataset_name + "/parquet/"
        parquet_dir = os.path.join(dataset_dir, 'parquet')
        s3_media_prefix = dataset_name + "/media/"
        media_dir = os.path.join(dataset_dir, 'media')
        s3_yml_prefix = dataset_name + "/yml/"
        yml_dir = os.path.join(dataset_dir, 'yml')
        remote_split_list = s3_client.list_splits(default_bucket, dataset_name)
        dataset_info_path = os.path.join(parquet_dir, 'dataset.yaml')

        # get the whole dataset
        if not split_name:
            label_exist_flag = db_client.is_dataset_label_downloaded(dataset_name)
            media_exist_flag = db_client.is_dataset_media_downloaded(dataset_name)

            # check whether the dataset locally exists
            if dataset_exist_flag:
                if label_exist_flag and media_exist_flag:
                    reminder = 'The dataset "%s" has already been all downloaded in %s.' % (dataset_name, dataset_dir)
                    print(reminder)
                    exit()

            # download a new dataset
            if not dataset_exist_flag:
                if not os.path.exists(dataset_dir):
                    os.mkdir(dataset_dir)
                print("saving to %s" % dataset_dir)

                # download dataset
                if not label_flag:
                    s3_client.download_directory(default_bucket, dataset_name + '/', dataset_dir)
                else:
                    print("parquet folder: ")
                    s3_client.download_directory(default_bucket, s3_parquet_prefix, parquet_dir)
                    print("yml folder: ")
                    s3_client.download_directory(default_bucket, s3_yml_prefix, yml_dir)
                print("register local dataset...")

                # get meta info of dataset to insert into sqlite
                parquet_list = [obj['Key'].split("/")[-1] for obj in
                                s3_client.list_objects(default_bucket, dataset_name + '/parquet/') if
                                str(obj['Key']).endswith(".parquet")]

                with open(dataset_info_path, 'r') as f:
                    stat = yaml.safe_load(f)['statistics']

                dataset_media_num = stat['dataset_stat']['media_num']
                dataset_media_size = stat['dataset_stat']['media_size']

                if not label_flag:
                    db_client.register_dataset(dataset_name, output, dataset_dir, 1, 1, dataset_media_num,
                                               dataset_media_size)

                    # get meta info of split to insert into sqlite
                    for split in parquet_list:
                        stat = query.ParquetReader(os.path.join(dataset_dir, 'parquet', split)).get_metadata()
                        split_media_num = stat['split_stat']['media_num']
                        split_media_size = stat['split_stat']['media_size']
                        db_client.register_split(dataset_name, split.replace(".parquet", ""), 'official', 1, 1,
                                                 split_media_num,
                                                 split_media_size)
                else:
                    db_client.register_dataset(dataset_name, output, dataset_dir, 1, 0, dataset_media_num,
                                               dataset_media_size)

                    # get meta info of split to insert into sqlite
                    for split in parquet_list:
                        stat = query.ParquetReader(os.path.join(dataset_dir, 'parquet', split)).get_metadata()
                        split_media_num = stat['split_stat']['media_num']
                        split_media_size = stat['split_stat']['media_size']
                        db_client.register_split(dataset_name, split.replace(".parquet", ""), 'official', 1, 0,
                                                 split_media_num,
                                                 split_media_size)

            # dataset has already been partly downloaded
            else:
                print("check label data...")
                if label_exist_flag:
                    print("label data has been all downloaded")
                else:
                    download_split_list = [split_name for split_name in remote_split_list if
                                           not db_client.is_split_local_exist(dataset_name, split_name)]
                    download_file_list = [split_name + '.parquet' for split_name in download_split_list]
                    if len(download_file_list) == 0:
                        print("label data has been all downloaded")
                    else:
                        print("download missing label files")
                        s3_client.download_list(default_bucket, download_file_list, s3_parquet_prefix, parquet_dir)

                        print("register local split...")
                        for split in download_file_list:
                            stat = query.ParquetReader(os.path.join(parquet_dir, split)).get_metadata()
                            split_media_num = stat['split_stat']['media_num']
                            split_media_size = stat['split_stat']['media_size']
                            db_client.register_split(dataset_name, split.replace(".parquet", ""), 'official', 1, 1,
                                                     split_media_num,
                                                     split_media_size)

                print("check media data...")
                if media_exist_flag:
                    print("media data has been all downloaded")
                else:
                    if not os.path.exists(media_dir):
                        os.mkdir(media_dir)
                    print("download missing media files")
                    s3_client.download_directory(default_bucket, s3_media_prefix, media_dir)

                s3_client.download_directory(default_bucket, s3_yml_prefix, yml_dir)

                print("update dataset info...")
                db_client.cursor.execute(
                    "update dataset set label_data=?, media_data=?, updated_time=datetime('now','localtime') where dataset_name=?",
                    [1, 1, dataset_name])
                db_client.cursor.execute(
                    "update split set label_data=?, media_data=?, updated_time=datetime('now','localtime') where dataset_name=?",
                    [1, 1, dataset_name])
                db_client.conn.commit()

        # download a split of dataset
        else:
            if split_name in remote_split_list:

                s3_parquet_key = s3_parquet_prefix + split_name + '.parquet'
                s3_datainfo_key = s3_parquet_prefix + 'dataset.yaml'
                parquet_path = db_client.get_local_split_path(dataset_name, split_name) \
                    if split_exist_flag else os.path.join(parquet_dir, split_name + '.parquet')

                if not split_exist_flag:
                    if not os.path.exists(dataset_dir):
                        os.mkdir(dataset_dir)
                    if not os.path.exists(parquet_dir):
                        os.mkdir(parquet_dir)

                    s3_client.download_file(default_bucket, s3_parquet_key, parquet_path)
                    s3_client.download_file(default_bucket, s3_datainfo_key, dataset_info_path)
                    parquet_reader = query.ParquetReader(parquet_path)
                    s3_media_keys = parquet_reader.select('image')['image'].tolist()

                    if not label_flag:
                        if not os.path.exists(media_dir):
                            os.mkdir(media_dir)

                        s3_client.download_list(default_bucket, s3_media_keys, dataset_name + "/", dataset_dir)

                    print("register local split...")
                    if not dataset_exist_flag:
                        with open(dataset_info_path, 'r') as f:
                            stat = yaml.safe_load(f)['statistics']
                        dataset_media_num = stat['dataset_stat']['media_num']
                        dataset_media_size = stat['dataset_stat']['media_size']
                        db_client.register_dataset(dataset_name, output, dataset_dir, 0, 0, dataset_media_num,
                                                   dataset_media_size)

                        stat = query.ParquetReader(parquet_path).get_metadata()
                        split_media_num = stat['split_stat']['media_num']
                        split_media_size = stat['split_stat']['media_size']
                        if label_flag:
                            db_client.register_split(dataset_name, split_name, 'official', 1, 0, split_media_num,
                                                     split_media_size)
                        else:
                            db_client.register_split(dataset_name, split_name, 'official', 1, 1, split_media_num,
                                                     split_media_size)
                    else:
                        stat = query.ParquetReader(parquet_path).get_metadata()
                        split_media_num = stat['split_stat']['media_num']
                        split_media_size = stat['split_stat']['media_size']
                        if label_flag:
                            db_client.register_split(dataset_name, split_name, 'official', 1, 0, split_media_num,
                                                     split_media_size)
                        else:
                            db_client.register_split(dataset_name, split_name, 'official', 1, 1, split_media_num,
                                                     split_media_size)

                else:
                    local_db_split_dict = db_client.get_sqlite_dict_list(
                        "select * from split where dataset_name='%s' and split_name='%s'" % (
                            dataset_name, split_name))[0]
                    label_data = local_db_split_dict['label_data']
                    media_data = local_db_split_dict['media_data']
                    if label_data == 1 and media_data == 1:
                        print("the split has been fully downloaded")
                        exit()
                    elif label_data == 1 and media_data == 0:
                        if label_flag:
                            print("label data of the split has been downloaded")
                        else:
                            if not os.path.exists(media_dir):
                                os.mkdir(media_dir)
                            parquet_reader = query.ParquetReader(parquet_path)
                            s3_media_keys = parquet_reader.select('image')['image'].tolist()
                            s3_client.download_list(default_bucket, s3_media_keys, dataset_name + "/", dataset_dir)
                            print("update split info...")
                            db_client.cursor.execute(
                                "update split set media_data=?, updated_time=datetime('now','localtime') where dataset_name=? and split_name=?",
                                [1, dataset_name, split_name])
                            db_client.conn.commit()

            else:
                if not split_exist_flag:
                    print(
                        "can not find the split named %s of dataset %s neither in remote repo nor in local storage" % (
                        split_name, dataset_name))
                    exit()

                parquet_path = db_client.get_local_split_path(dataset_name, split_name)

                local_db_split_dict = db_client.get_sqlite_dict_list(
                    "select * from split where dataset_name='%s' and split_name='%s'" % (
                        dataset_name, split_name))[0]
                label_data = local_db_split_dict['label_data']
                media_data = local_db_split_dict['media_data']
                if label_data == 1 and media_data == 1:
                    print("the split has been fully downloaded")
                    exit()
                elif label_data == 1 and media_data == 0:
                    if label_flag:
                        print("label data of the split has been downloaded")
                    else:
                        if not os.path.exists(media_dir):
                            os.mkdir(media_dir)
                        parquet_reader = query.ParquetReader(parquet_path)
                        s3_media_keys = parquet_reader.select('image')['image'].tolist()
                        s3_client.download_list(default_bucket, s3_media_keys, dataset_name + "/", dataset_dir)
                        print("update split info...")
                        db_client.cursor.execute(
                            "update split set media_data=?, updated_time=datetime('now','localtime') where dataset_name=? and split_name=?",
                            [1, dataset_name, split_name])
                        db_client.conn.commit()



class SDKException(Exception):

    def __init__(self, message):
        super().__init__(message)


class DatasetPathNotExists(SDKException):

    def __init__(self, message):
        super().__init__(message)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cli',
 'cli.commands',
 'cli.commons',
 'cli.utils',
 'cli.utils.minio_ops',
 'cli.utils.oss_ops',
 'cli.utils.views',
 'cli.utils.views.cifar-10',
 'cli.utils.views.cifar-10-auto',
 'cli.utils.views.coco-demo',
 'cli.utils.views.coco-demo.src',
 'cli.utils.views.coco-demo.src.constants',
 'cli.utils.views.coco-demo.src.utils',
 'dsdlsdk',
 'dsdlsdk.exception']

package_data = \
{'': ['*'], 'cli': ['resources/*'], 'dsdlsdk': ['common/*']}

install_requires = \
['Pillow>=9.1.0,<10.0.0',
 'boto3>=1.26.14,<2.0.0',
 'click>=8.1.3,<9.0.0',
 'duckdb>=0.6.0,<0.7.0',
 'jinja2>=3.0.0,<4.0.0',
 'jsonmodels>=2.6.0,<3.0.0',
 'loguru>=0.6.0,<0.7.0',
 'networkx>=2.8.4,<3.0.0',
 'opencv-python>=4.6.0.66,<5.0.0.0',
 'oss2>=2.15.0,<3.0.0',
 'pandas>=1.5.1,<2.0.0',
 'prettytable>=3.3.0,<4.0.0',
 'psutil>=5.9.4,<6.0.0',
 'pyarrow>=10.0.0,<11.0.0',
 'pyyaml>=6.0,<7.0',
 'rich>=12.6.0,<13.0.0',
 'setuptools-scm>=7.0.5,<8.0.0',
 'smart-open[all]>=6.2.0,<7.0.0',
 'stqdm>=0.0.4,<0.0.5',
 'streamlit>=1.15.1,<2.0.0',
 'tabulate>=0.9.0,<0.10.0',
 'termgraph>=0.5.3,<0.6.0',
 'toml>=0.10.2,<0.11.0',
 'tqdm>=4.64.1,<5.0.0']

entry_points = \
{'console_scripts': ['dsdl = dsdl.cli:cli', 'odl-cli = cli.cli:main']}

setup_kwargs = {
    'name': 'odl-cli',
    'version': '0.1.1.post150',
    'description': 'made for opendatalab dsdl-sdk dev-cli branch(ignore other branches).',
    'long_description': '<div align="center">\n  <img src="resources/opendatalab.svg" width="600"/>\n  <div>&nbsp;</div>\n  <div align="center">\n    <b><font size="5">OpenDataLab website</font></b>\n    <sup>\n      <a href="https://opendatalab.com/">\n        <i><font size="4">HOT</font></i>\n      </a>\n    </sup>\n    &nbsp;&nbsp;&nbsp;&nbsp;\n  </div>\n  <div>&nbsp;</div>\n</div>\n\nEnglish | [简体中文](README_zh-CN.md)\n\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/dsdl) ](https://pypi.org/project/dsdl/)[![PyPI](https://img.shields.io/pypi/v/dsdl)](https://pypi.org/project/dsdl) [![docs](https://img.shields.io/badge/docs-latest-blue)](https://opendatalab.github.io/dsdl-docs/)[![dev workflow](https://github.com/opendatalab/dsdl-sdk/actions/workflows/dev.yml/badge.svg?branch=dev)](https://github.com/opendatalab/dsdl-sdk/actions/workflows/dev.yml)[![stage & preview workflow](https://github.com/opendatalab/dsdl-sdk/actions/workflows/preview.yml/badge.svg?branch=dev)](https://github.com/opendatalab/dsdl-sdk/actions/workflows/preview.yml)\n\n[📘Documentation](https://opendatalab.github.io/dsdl-docs/) |\n\n## Introduction\n\n**Data** is the cornerstone of artificial intelligence. The efficiency of data acquisition, exchange, and application directly impacts the advances in technologies and applications. Over the long history of AI, a vast quantity of data sets have been developed and distributed. However, these datasets are defined in very different forms, which incurs significant overhead when it comes to exchange, integration, and utilization -- it is often the case that one needs to develop a new customized tool or script in order to incorporate a new dataset into a workflow.\n\nTo overcome such difficulties, we develop **Data Set Description Language (DSDL)**.\n\n### Major features\n\nThe design of **DSDL** is driven by three goals, namely *generic*, *portable*, *extensible*. We refer to these three goals together as **GPE**.\n\n* **Generic**\n\n  This language aims to provide a unified representation standard for data in multiple fields of artificial intelligence, rather than being designed for a single field or task. It should be able to express data sets with different modalities and structures in a consistent format.\n\n* **Portable**\n\n  Write once, distribute everywhere. Dataset descriptions can be widely distributed and exchanged, and used in different environments without modification of the source files. The achievement of this goal is crucial for creating an open and thriving ecosystem. To this end, we need to carefully examine the details of the design, and remove unnecessary dependencies on specific assumptions about the underlying facilities or organizations.\n\n* **Extensible**\n\n  One should be able to extend the boundary of expression without modifying the core standard. For a programming language such as C++ or Python, its application boundaries can be significantly extended by libraries or packages, while the core language remains stable over a long period. Such libraries and packages form a rich ecosystem, making the language stay alive for a very long time.\n\n## Installation\n\n**Case a** install it with pip\n\n```bash\npip install dsdl\n```\n\n**Case b** install it from source\n\n```shell\ngit clone https://github.com/opendatalab/dsdl.git\ncd dsdl\npython setup.py install\n```\n\n## Get Started\n\n#### Use dsdl parser to deserialize the Yaml file to Python code\n```bash\ndsdl parse --yaml demo/coco_demo.yaml\n```\n\n#### Modify the configuration & set the directory of media in dataset\n\nCreate a configuration file `config.py` with the following contents（for now dsdl only reading from aliyun oss or local is supported）：\n\n```python\nlocal = dict(\n    type="LocalFileReader",\n    working_dir="local path of your media",\n)\n\nali_oss = dict(\n    type="AliOSSFileReader",\n    access_key_secret="your secret key of aliyun oss",\n    endpoint="your endpoint of aliyun oss",\n    access_key_id="your access key of aliyun oss",\n    bucket_name="your bucket name of aliyun oss",\n    working_dir="the relative path of your media dir in the bucket")\n```\n\n In `config.py`, the configuration of how to read the media in a dataset is defined. One should specify the arguments depending on from where to read the media：  \n\n1. read from local： `working_dir` field in `local` should be specified (the directory of local media)    \n2. read from aliyun oss： all the field in `ali_oss `should be specified (including `access_key_secret`, `endpoint`, `access_key_id`, `bucket_name`, `working_dir`)  \n\n#### Visualize samples\n\n   ```bash\n   dsdl view -y <yaml-name>.yaml -c <config.py> -l ali-oss -n 10 -r -v -f Label BBox Attributes\n   ```\n\nThe description of each argument is shown below:  \n\n| simplified  argument | argument      | description                                                  |\n| -------------------- | ------------- | :----------------------------------------------------------- |\n| -y                   | `--yaml`      | The path of dsdl yaml file.                                  |\n| -c                   | `--config`    | The path of  location configuration file.                    |\n| -l                   | `--location`  | `local` or `ali-oss`，which means read media from local or aliyun oss. |\n| -n                   | `--num`       | The number of samples to be visualized.                      |\n| -r                   | `--random`    | Whether to load the samples in a random order.               |\n| -v                   | `--visualize` | Whether to visualize the samples or just print the information in console. |\n| -f                   | `--field`     | The field type to visualize, e.g. `-f BBox`means show the bounding box in samples, `-f Attributes`means show the attributes of a sample in the console . One can specify multiple field types simultaneously, such as `-f Label BBox  Attributes`. |\n| -t                   | `--task`      | The task you are working on, for example, `-t detection` is equivalent to `-f Label BBox Polygon Attributes`. |\n\n## Citation\n\nIf you find this project useful in your research, please consider cite:\n\n```bibtex\n@misc{dsdl2022,\n    title={{DSDL}: Data Set Description Language},\n    author={DSDL Contributors},\n    howpublished = {\\url{https://github.com/opendatalab/dsdl}},\n    year={2022}\n}\n```\n\n## License\n\n`DSDL` is released under the [Apache 2.0 license](LICENSE).\n\n## Acknowledgement\n\n* Field & Model Design inspired by [Django ORM](https://www.djangoproject.com/) and [jsonmodels](https://github.com/jazzband/jsonmodels)\n',
    'author': 'odl-cli',
    'author_email': 'odl-cli@pjlab.org.cn',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/opendatalab/dsdl-sdk/tree/dev-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8, !=2.7.*, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*, !=3.6.*, !=3.7.*',
}


setup(**setup_kwargs)



"""
一个命令的实现例子

Examples:
    $python cli.py example --show tell me the truth
    >> Namespace(show=["'tell", 'me', 'the', "truth'"], command_handler=<bound method Example.cmd_entry of <commands.example.Example object at 0x0000017E6FD1DB40>>)
    >> ["'tell", 'me', 'the', "truth'"]
"""
import os

from commands.cmdbase import CmdBase
from commands.const import DSDL_CLI_DATASET_NAME, DEFAULT_LOCAL_STORAGE_PATH
from commons.argument_parser import EnvDefaultVar
from utils import admin, query
from utils.oss_ops import ops
import yaml

aws_access_key_id = query.aws_access_key_id
aws_secret_access_key = query.aws_secret_access_key
endpoint_url = query.endpoint_url
region_name = query.region_name
default_bucket = query.default_bucket

default_path = DEFAULT_LOCAL_STORAGE_PATH


class Select(CmdBase):
    """
    Example command
    """

    def init_parser(self, subparsers):
        """
        Initialize the parser for the command
        document : https://docs.python.org/zh-cn/3/library/argparse.html#

        Args:
            subparsers:

        Returns:

        """
        select_parser = subparsers.add_parser('select', help='Select data from dataset',
                                              example="select.example",
                                              description='Select data from dataset', )  # example 样例文件位于resources/下，普通的文本文件，每个命令写一个
        # select_parser.add_argument("-s", '--show', nargs='?', default='SHOW', help='show example string.....',
        #                            metavar='SS')
        # example_parser.add_argument('--test1', nargs='?', default='x', help='show test1', metavar='a')
        # example_parser.add_argument('--test-1234', nargs='?', default='t', help='show test-1234', metavar='c')
        select_parser.add_argument("dataset_name", action=EnvDefaultVar, envvar=DSDL_CLI_DATASET_NAME,
                                   type=str,
                                   help='Dataset name. The arg is optional only when the default dataset name was set by cd command.',
                                   metavar='')
        select_parser.add_argument("--split-name", type=str, required=True,
                                   help='The split name of the dataset, such as train/test/unlabeled or user self-defined split.',
                                   metavar='')
        select_parser.add_argument("--filter", type=str,
                                   help='Filter data according to given conditions, such as "label=\'bird\'"',
                                   metavar='')
        # select_parser.add_argument("--fields", type=str,
        #                            help='Fields to be selected. All the files will be selected if the arg is not given. Use "," to split fields.',
        #                            metavar='')
        select_parser.add_argument("--limit", type=int,
                                   help='Limit the number of returned records',
                                   metavar='')
        select_parser.add_argument("--offset", type=int,
                                   help='Set the number of rows to skip from the beginning of the returned data before presenting the results',
                                   metavar='')
        select_parser.add_argument("--random", type=int,
                                   # help='Set the number/percent of random samples from the base select result, such as 100 or 5%',
                                   help='Set the number of random samples from the returned select result',
                                   metavar='')
        select_parser.add_argument("--export-name", type=str,
                                   help='Save the select result as a split and use the given name to name it',
                                   metavar='')
        select_parser.add_argument("--output", type=str,
                                   help='The path to save the select result as a new split',
                                   metavar='')
        return select_parser

    def cmd_entry(self, cmdargs, config, *args, **kwargs):
        """
        Entry point for the command

        Args:
            self:
            cmdargs:
            config:

        Returns:

        """
        conf_dict = admin.get_config_dict()

        dataset_name = cmdargs.dataset_name
        split_name = cmdargs.split_name
        filter = cmdargs.filter
        # fields = '*' if cmdargs.fields is None else cmdargs.fields
        fields = '*'
        limit = cmdargs.limit
        offset = cmdargs.offset
        random = cmdargs.random
        export_name = cmdargs.export_name
        output = cmdargs.output if cmdargs.output else 'default'
        output_path = conf_dict['storage'][cmdargs.output]['path'] if cmdargs.output else default_path

        db_client = admin.DBClient()
        s3_client = ops.OssClient(aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,
                                  endpoint_url=endpoint_url, region_name=region_name)

        # if not db_client.is_dataset_local_exist(dataset_name):
        #     print("there is no dataset named %s locally" % dataset_name)
        #     exit()

        # assert (db_client.is_dataset_local_exist(dataset_name)), "there is no dataset named %s locally" % dataset_name

        if not db_client.is_split_local_exist(dataset_name, split_name):
            print(
                "there is no split named %s of dataset %s locally, search in remote repo" % (split_name, dataset_name))

            if not s3_client.is_split_remote_exist(default_bucket, dataset_name, split_name):
                print("there is no split named %s of dataset %s in remote repo" % (split_name, dataset_name))
                exit()

        split_reader = query.SplitReader(dataset_name, split_name)
        df = split_reader.select(select_cols=fields, filter_cond=filter, limit=limit, offset=offset, samples=random)
        print(df)

        if export_name is not None:
            sub_split = query.Split(dataset_name, export_name, output_path)
            media_download_flag = 0
            question = 'The split "%s" has already existed. Do you want to replace it? (y/n)' % export_name
            if sub_split.is_local_exist():
                if not input(question) in ("y", "Y"):
                    exit()

            if db_client.is_dataset_local_exist(dataset_name):
                path_info = "the dataset %s registered locally, the split has to been exported to %s" % (
                    dataset_name, sub_split.parquet_path)
                split_db_info = db_client.get_sqlite_dict_list(
                    "select * from split where dataset_name='%s' and split_name='%s'" % (dataset_name, split_name))
                if len(split_db_info) != 0:
                    # print(split_db_info)
                    media_download_flag = split_db_info[0]['media_data']

                if media_download_flag:
                    media_path = [os.path.join(sub_split.dataset_path, x) for x in df['image'].tolist()]
                    media_stat = admin.get_media_stat(media_path)
                    media_num = media_stat['media_num']
                    media_size = media_stat['media_size']

                    split_stat = {'media_num': media_num, 'media_size': media_size}

                    stat = split_reader.get_metadata()
                    stat['split_stat'] = split_stat

                    schema = split_reader.get_schema()

                else:
                    media_prefix = "%s/" % dataset_name
                    media_path = [media_prefix + x for x in df['image'].tolist()]
                    media_num = len(set(media_path))
                    media_size = s3_client.get_sum_size(default_bucket, media_path)

                    split_stat = {'media_num': media_num, 'media_size': media_size}
                    stat = split_reader.get_metadata()
                    stat['split_stat'] = split_stat

                    schema = split_reader.get_schema()

            else:
                path_info = "the dataset %s does not exist locally, the split has to been exported to given output %s" % (
                    dataset_name, sub_split.parquet_path)

                media_prefix = "%s/" % dataset_name
                media_path = [media_prefix + x for x in df['image'].tolist()]
                media_num = len(set(media_path))
                media_size = s3_client.get_sum_size(default_bucket, media_path)

                split_stat = {'media_num': media_num, 'media_size': media_size}
                stat = split_reader.get_metadata()
                stat['split_stat'] = split_stat

                schema = split_reader.get_schema()

                yaml_key = '/'.join([dataset_name, 'parquet', 'dataset.yaml'])
                dataset_dict = yaml.safe_load(s3_client.read_file(default_bucket, yaml_key))
                dataset_media_num = dataset_dict["statistics"]["dataset_stat"]['media_num']
                dataset_media_size = dataset_dict["statistics"]["dataset_stat"]['media_size']

                db_client.register_dataset(dataset_name, output, sub_split.dataset_path, 0, 0, dataset_media_num,
                                           dataset_media_size)

            sub_split.save(df, schema, 'user-defined', 1, media_download_flag, stat)
            print(path_info)

            # to do operations about fields
            # a, b = query.get_parquet_metadata(parquet_path)
            # print(a)
            # print(b)
            #
            # for n in query.get_parquet_schema(parquet_path):
            #     print(n)
